@@ -20,6 +20,7 @@
 
 #include <config.h>
 
+#include <float.h>
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -47,6 +48,7 @@ enum {
         PROP_0,
         PROP_MANAGER,
         PROP_REAL_NAME,
+        PROP_DISPLAY_NAME,
         PROP_USER_NAME,
         PROP_UID,
         PROP_HOME_DIR,
@@ -68,6 +70,7 @@ struct _GdmUser {
         uid_t           uid;
         char           *user_name;
         char           *real_name;
+        char           *display_name;
         char           *home_dir;
         char           *shell;
         GList          *sessions;
@@ -206,6 +209,9 @@ gdm_user_get_property (GObject    *object,
         case PROP_REAL_NAME:
                 g_value_set_string (value, user->real_name);
                 break;
+        case PROP_DISPLAY_NAME:
+                g_value_set_string (value, user->display_name);
+                break;
         case PROP_HOME_DIR:
                 g_value_set_string (value, user->home_dir);
                 break;
@@ -252,6 +258,14 @@ gdm_user_class_init (GdmUserClass *class)
                                                               NULL,
                                                               G_PARAM_READABLE));
 
+        g_object_class_install_property (gobject_class,
+                                         PROP_DISPLAY_NAME,
+                                         g_param_spec_string ("display-name",
+                                                              "Display Name",
+                                                              "The unique name to display for this user.",
+                                                              NULL,
+                                                              G_PARAM_READABLE));
+
         g_object_class_install_property (gobject_class,
                                          PROP_UID,
                                          g_param_spec_ulong ("uid",
@@ -369,6 +383,7 @@ gdm_user_init (GdmUser *user)
         user->manager = NULL;
         user->user_name = NULL;
         user->real_name = NULL;
+        user->display_name = NULL;
         user->sessions = NULL;
 }
 
@@ -383,6 +398,7 @@ gdm_user_finalize (GObject *object)
 
         g_free (user->user_name);
         g_free (user->real_name);
+        g_free (user->display_name);
 
         if (G_OBJECT_CLASS (gdm_user_parent_class)->finalize)
                 (*G_OBJECT_CLASS (gdm_user_parent_class)->finalize) (object);
@@ -411,13 +427,16 @@ _gdm_user_update (GdmUser             *user,
         /* Display Name */
         if (pwent->pw_gecos && pwent->pw_gecos[0] != '\0') {
                 gchar *first_comma;
+                gchar *real_name_utf8;
 
-                first_comma = strchr (pwent->pw_gecos, ',');
+                real_name_utf8 = g_locale_to_utf8 (pwent->pw_gecos, -1, NULL, NULL, NULL);
+
+                first_comma = strchr (real_name_utf8, ',');
                 if (first_comma) {
-                        real_name = g_strndup (pwent->pw_gecos,
-                                                  (first_comma - pwent->pw_gecos));
+                        real_name = g_strndup (real_name_utf8, first_comma - real_name_utf8);
+                        g_free (real_name_utf8);
                 } else {
-                        real_name = g_strdup (pwent->pw_gecos);
+                        real_name = real_name_utf8;
                 }
 
                 if (real_name[0] == '\0') {
@@ -440,6 +459,16 @@ _gdm_user_update (GdmUser             *user,
                 g_free (real_name);
         }
 
+        /* Unique Display Name */
+        if ((!user->real_name && user->display_name) ||
+            (user->real_name &&
+             user->display_name &&
+             strncmp (user->real_name, user->display_name, strlen (user->real_name)) != 0)) {
+                g_free (user->display_name);
+                user->display_name = NULL;
+                g_object_notify (G_OBJECT (user), "display-name");
+        }
+
         /* UID */
         if (pwent->pw_uid != user->uid) {
                 user->uid = pwent->pw_uid;
@@ -538,6 +567,26 @@ gdm_user_get_real_name (GdmUser *user)
         return (user->real_name ? user->real_name : user->user_name);
 }
 
+/**
+ * gdm_user_get_display_name:
+ * @user: the user object to examine.
+ *
+ * Retrieves the unique display name of @user.
+ *
+ * Returns: a pointer to an array of characters which must not be modified or
+ *  freed, or %NULL.
+ *
+ * Since: 1.0
+ **/
+G_CONST_RETURN gchar *
+gdm_user_get_display_name (GdmUser *user)
+{
+        g_return_val_if_fail (GDM_IS_USER (user), NULL);
+
+        return (user->display_name ? user->display_name
+                : gdm_user_get_real_name (user));
+}
+
 /**
  * gdm_user_get_user_name:
  * @user: the user object to examine.
@@ -606,6 +655,62 @@ gdm_user_get_login_frequency (GdmUser *user)
         return user->login_frequency;
 }
 
+/**
+ * _gdm_user_show_full_display_name:
+ * @user: the user object to examine.
+ *
+ * Updates the unique display name of @user to "Real Name (username)".
+ *
+ * Since: 1.0
+ **/
+void
+_gdm_user_show_full_display_name (GdmUser *user)
+{
+        char *uniq_name;
+
+        g_return_if_fail (GDM_IS_USER (user));
+
+        if (user->real_name != NULL) {
+                uniq_name = g_strdup_printf ("%s (%s)",
+                                             user->real_name,
+                                             user->user_name);
+        } else {
+                uniq_name = NULL;
+        }
+
+        if ((user->real_name && !user->display_name) ||
+            (!user->real_name && user->display_name) ||
+            (user->real_name &&
+             user->display_name &&
+             strcmp (uniq_name, user->display_name) != 0)) {
+                g_free (user->display_name);
+                user->display_name = uniq_name;
+                g_object_notify (G_OBJECT (user), "display-name");
+        } else {
+                g_free (uniq_name);
+        }
+}
+
+/**
+ * _gdm_user_show_short_display_name:
+ * @user: the user object to examine.
+ *
+ * Resets the unique display name of @user.
+ *
+ * Since: 1.0
+ **/
+void
+_gdm_user_show_short_display_name (GdmUser *user)
+{
+        g_return_if_fail (GDM_IS_USER (user));
+
+        if (user->display_name) {
+                g_free (user->display_name);
+                user->display_name = NULL;
+                g_object_notify (G_OBJECT (user), "display-name");
+        }
+}
+
 int
 gdm_user_collate (GdmUser *user1,
                   GdmUser *user2)
@@ -703,62 +808,74 @@ check_user_file (const char *filename,
         return TRUE;
 }
 
+static char *
+get_filesystem_type (const char *path)
+{
+        GFile      *file;
+        GFileInfo  *file_info;
+        GError     *error;
+        char       *filesystem_type;
+
+        file = g_file_new_for_path (path);
+        error = NULL;
+        file_info = g_file_query_filesystem_info (file,
+                                                  G_FILE_ATTRIBUTE_FILESYSTEM_TYPE,
+                                                  NULL,
+                                                  &error);
+        if (file_info == NULL) {
+                g_warning ("Unable to query filesystem type for %s: %s", path, error->message);
+                g_error_free (error);
+                g_object_unref (file);
+                return NULL;
+        }
+
+        filesystem_type = g_strdup (g_file_info_get_attribute_string (file_info,
+                                                                      G_FILE_ATTRIBUTE_FILESYSTEM_TYPE));
+        if (filesystem_type == NULL) {
+                g_warning ("GIO returned NULL filesystem type for %s", path);
+        }
+
+        g_object_unref (file);
+        g_object_unref (file_info);
+
+        return filesystem_type;
+}
+
 static GdkPixbuf *
 render_icon_from_home (GdmUser *user,
                        int      icon_size)
 {
         GdkPixbuf  *retval;
         char       *path;
-        GFile      *file;
-        GFileInfo  *file_info;
         gboolean    is_local;
         gboolean    is_autofs;
         gboolean    res;
-        const char *filesystem_type;
+        char       *filesystem_type;
+
+        is_local = FALSE;
 
         /* special case: look at parent of home to detect autofs
            this is so we don't try to trigger an automount */
         path = g_path_get_dirname (user->home_dir);
-        file = g_file_new_for_path (path);
-        file_info = g_file_query_filesystem_info (file,
-                                                  G_FILE_ATTRIBUTE_FILESYSTEM_TYPE,
-                                                  NULL,
-                                                  NULL);
-        if (file_info == NULL) {
-                g_free (path);
-                g_object_unref (file);
-                return NULL;
-        }
-        filesystem_type = g_file_info_get_attribute_string (file_info,
-                                                            G_FILE_ATTRIBUTE_FILESYSTEM_TYPE);
+        filesystem_type = get_filesystem_type (path);
         is_autofs = (filesystem_type != NULL && strcmp (filesystem_type, "autofs") == 0);
-        g_object_unref (file);
-        g_object_unref (file_info);
+        g_free (filesystem_type);
         g_free (path);
 
-        /* now check that home dir itself is local */
-        is_local = TRUE;
-        if (! is_autofs) {
-                file = g_file_new_for_path (user->home_dir);
-                file_info = g_file_query_filesystem_info (file,
-                                                          G_FILE_ATTRIBUTE_FILESYSTEM_TYPE,
-                                                          NULL,
-                                                          NULL);
-                if (file_info == NULL) {
-                        g_object_unref (file);
-                        return NULL;
-                }
-                filesystem_type = g_file_info_get_attribute_string (file_info,
-                                                                    G_FILE_ATTRIBUTE_FILESYSTEM_TYPE);
-                is_local = ((strcmp (filesystem_type, "nfs") != 0) &&
-                            (strcmp (filesystem_type, "afs") != 0) &&
-                            (strcmp (filesystem_type, "autofs") != 0) &&
-                            (strcmp (filesystem_type, "unknown") != 0) &&
-                            (strcmp (filesystem_type, "ncpfs") != 0));
-                g_object_unref (file_info);
-                g_object_unref (file);
+        if (is_autofs) {
+                return NULL;
         }
 
+        /* now check that home dir itself is local */
+        filesystem_type = get_filesystem_type (user->home_dir);
+        is_local = ((filesystem_type != NULL) &&
+                    (strcmp (filesystem_type, "nfs") != 0) &&
+                    (strcmp (filesystem_type, "afs") != 0) &&
+                    (strcmp (filesystem_type, "autofs") != 0) &&
+                    (strcmp (filesystem_type, "unknown") != 0) &&
+                    (strcmp (filesystem_type, "ncpfs") != 0));
+        g_free (filesystem_type);
+
         /* only look at local home directories so we don't try to
            read from remote (e.g. NFS) volumes */
         if (! is_local) {
@@ -855,11 +972,243 @@ render_icon_from_home (GdmUser *user,
         return retval;
 }
 
+static void
+curved_rectangle (cairo_t *cr,
+                  double   x0,
+                  double   y0,
+                  double   width,
+                  double   height,
+                  double   radius)
+{
+        double x1;
+        double y1;
+
+        x1 = x0 + width;
+        y1 = y0 + height;
+
+        if (width < FLT_EPSILON || height < FLT_EPSILON) {
+                return;
+        }
+
+        if (width / 2 < radius) {
+                if (height / 2 < radius) {
+                        cairo_move_to  (cr, x0, (y0 + y1) / 2);
+                        cairo_curve_to (cr, x0 ,y0, x0, y0, (x0 + x1) / 2, y0);
+                        cairo_curve_to (cr, x1, y0, x1, y0, x1, (y0 + y1) / 2);
+                        cairo_curve_to (cr, x1, y1, x1, y1, (x1 + x0) / 2, y1);
+                        cairo_curve_to (cr, x0, y1, x0, y1, x0, (y0 + y1) / 2);
+                } else {
+                        cairo_move_to  (cr, x0, y0 + radius);
+                        cairo_curve_to (cr, x0, y0, x0, y0, (x0 + x1) / 2, y0);
+                        cairo_curve_to (cr, x1, y0, x1, y0, x1, y0 + radius);
+                        cairo_line_to (cr, x1, y1 - radius);
+                        cairo_curve_to (cr, x1, y1, x1, y1, (x1 + x0) / 2, y1);
+                        cairo_curve_to (cr, x0, y1, x0, y1, x0, y1 - radius);
+                }
+        } else {
+                if (height / 2 < radius) {
+                        cairo_move_to  (cr, x0, (y0 + y1) / 2);
+                        cairo_curve_to (cr, x0, y0, x0 , y0, x0 + radius, y0);
+                        cairo_line_to (cr, x1 - radius, y0);
+                        cairo_curve_to (cr, x1, y0, x1, y0, x1, (y0 + y1) / 2);
+                        cairo_curve_to (cr, x1, y1, x1, y1, x1 - radius, y1);
+                        cairo_line_to (cr, x0 + radius, y1);
+                        cairo_curve_to (cr, x0, y1, x0, y1, x0, (y0 + y1) / 2);
+                } else {
+                        cairo_move_to  (cr, x0, y0 + radius);
+                        cairo_curve_to (cr, x0 , y0, x0 , y0, x0 + radius, y0);
+                        cairo_line_to (cr, x1 - radius, y0);
+                        cairo_curve_to (cr, x1, y0, x1, y0, x1, y0 + radius);
+                        cairo_line_to (cr, x1, y1 - radius);
+                        cairo_curve_to (cr, x1, y1, x1, y1, x1 - radius, y1);
+                        cairo_line_to (cr, x0 + radius, y1);
+                        cairo_curve_to (cr, x0, y1, x0, y1, x0, y1 - radius);
+                }
+        }
+
+        cairo_close_path (cr);
+}
+
+static cairo_surface_t *
+surface_from_pixbuf (GdkPixbuf *pixbuf)
+{
+        cairo_surface_t *surface;
+        cairo_t         *cr;
+
+        surface = cairo_image_surface_create (gdk_pixbuf_get_has_alpha (pixbuf) ?
+                                              CAIRO_FORMAT_ARGB32 : CAIRO_FORMAT_RGB24,
+                                              gdk_pixbuf_get_width (pixbuf),
+                                              gdk_pixbuf_get_height (pixbuf));
+        cr = cairo_create (surface);
+        gdk_cairo_set_source_pixbuf (cr, pixbuf, 0, 0);
+        cairo_paint (cr);
+        cairo_destroy (cr);
+
+        return surface;
+}
+
+/**
+ * go_cairo_convert_data_to_pixbuf:
+ * @src: a pointer to pixel data in cairo format
+ * @dst: a pointer to pixel data in pixbuf format
+ * @width: image width
+ * @height: image height
+ * @rowstride: data rowstride
+ *
+ * Converts the pixel data stored in @src in CAIRO_FORMAT_ARGB32 cairo format
+ * to GDK_COLORSPACE_RGB pixbuf format and move them
+ * to @dst. If @src == @dst, pixel are converted in place.
+ **/
+
+static void
+go_cairo_convert_data_to_pixbuf (unsigned char *dst,
+                                 unsigned char const *src,
+                                 int width,
+                                 int height,
+                                 int rowstride)
+{
+        int i,j;
+        unsigned int t;
+        unsigned char a, b, c;
+
+        g_return_if_fail (dst != NULL);
+
+#define MULT(d,c,a,t) G_STMT_START { t = (a)? c * 255 / a: 0; d = t;} G_STMT_END
+
+        if (src == dst || src == NULL) {
+                for (i = 0; i < height; i++) {
+                        for (j = 0; j < width; j++) {
+#if G_BYTE_ORDER == G_LITTLE_ENDIAN
+                                MULT(a, dst[2], dst[3], t);
+                                MULT(b, dst[1], dst[3], t);
+                                MULT(c, dst[0], dst[3], t);
+                                dst[0] = a;
+                                dst[1] = b;
+                                dst[2] = c;
+#else
+                                MULT(a, dst[1], dst[0], t);
+                                MULT(b, dst[2], dst[0], t);
+                                MULT(c, dst[3], dst[0], t);
+                                dst[3] = dst[0];
+                                dst[0] = a;
+                                dst[1] = b;
+                                dst[2] = c;
+#endif
+                                dst += 4;
+                        }
+                        dst += rowstride - width * 4;
+                }
+        } else {
+                for (i = 0; i < height; i++) {
+                        for (j = 0; j < width; j++) {
+#if G_BYTE_ORDER == G_LITTLE_ENDIAN
+                                MULT(dst[0], src[2], src[3], t);
+                                MULT(dst[1], src[1], src[3], t);
+                                MULT(dst[2], src[0], src[3], t);
+                                dst[3] = src[3];
+#else
+                                MULT(dst[0], src[1], src[0], t);
+                                MULT(dst[1], src[2], src[0], t);
+                                MULT(dst[2], src[3], src[0], t);
+                                dst[3] = src[0];
+#endif
+                                src += 4;
+                                dst += 4;
+                        }
+                        src += rowstride - width * 4;
+                        dst += rowstride - width * 4;
+                }
+        }
+#undef MULT
+}
+
+static void
+cairo_to_pixbuf (guint8    *src_data,
+                 GdkPixbuf *dst_pixbuf)
+{
+        unsigned char *src;
+        unsigned char *dst;
+        guint          w;
+        guint          h;
+        guint          rowstride;
+
+        w = gdk_pixbuf_get_width (dst_pixbuf);
+        h = gdk_pixbuf_get_height (dst_pixbuf);
+        rowstride = gdk_pixbuf_get_rowstride (dst_pixbuf);
+
+        dst = gdk_pixbuf_get_pixels (dst_pixbuf);
+        src = src_data;
+
+        go_cairo_convert_data_to_pixbuf (dst, src, w, h, rowstride);
+}
+
+static GdkPixbuf *
+frame_pixbuf (GdkPixbuf *source)
+{
+        GdkPixbuf       *dest;
+        cairo_t         *cr;
+        cairo_surface_t *surface;
+        guint            w;
+        guint            h;
+        guint            rowstride;
+        int              frame_width;
+        double           radius;
+        guint8          *data;
+
+        frame_width = 2;
+
+        w = gdk_pixbuf_get_width (source) + frame_width * 2;
+        h = gdk_pixbuf_get_height (source) + frame_width * 2;
+        radius = w / 3.0;
+
+        dest = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
+                               TRUE,
+                               8,
+                               w,
+                               h);
+        rowstride = gdk_pixbuf_get_rowstride (dest);
+
+
+        data = g_new0 (guint8, h * rowstride);
+
+        surface = cairo_image_surface_create_for_data (data,
+                                                       CAIRO_FORMAT_ARGB32,
+                                                       w,
+                                                       h,
+                                                       rowstride);
+        cr = cairo_create (surface);
+        cairo_surface_destroy (surface);
+
+        /* set up image */
+        cairo_rectangle (cr, 0, 0, w, h);
+        cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.0);
+        cairo_fill (cr);
+
+        curved_rectangle (cr, frame_width, frame_width,
+                          w - frame_width * 2, h - frame_width * 2,
+                          radius);
+        cairo_set_source_rgba (cr, 0.5, 0.5, 0.5, 0.3);
+        cairo_fill_preserve (cr);
+
+        surface = surface_from_pixbuf (source);
+        cairo_set_source_surface (cr, surface, frame_width, frame_width);
+        cairo_fill (cr);
+        cairo_surface_destroy (surface);
+
+        cairo_to_pixbuf (data, dest);
+
+        cairo_destroy (cr);
+        g_free (data);
+
+        return dest;
+}
+
 GdkPixbuf *
 gdm_user_render_icon (GdmUser   *user,
                       gint       icon_size)
 {
         GdkPixbuf    *pixbuf;
+        GdkPixbuf    *framed;
         char         *path;
         char         *tmp;
         gboolean      res;
@@ -867,9 +1216,11 @@ gdm_user_render_icon (GdmUser   *user,
         g_return_val_if_fail (GDM_IS_USER (user), NULL);
         g_return_val_if_fail (icon_size > 12, NULL);
 
+        path = NULL;
+
         pixbuf = render_icon_from_home (user, icon_size);
         if (pixbuf != NULL) {
-                return pixbuf;
+                goto out;
         }
 
         /* Try ${GlobalFaceDir}/${username} */
@@ -890,7 +1241,7 @@ gdm_user_render_icon (GdmUser   *user,
 
         g_free (path);
         if (pixbuf != NULL) {
-                return pixbuf;
+                goto out;
         }
 
         /* Finally, ${GlobalFaceDir}/${username}.png */
@@ -910,8 +1261,16 @@ gdm_user_render_icon (GdmUser   *user,
         } else {
                 pixbuf = NULL;
         }
-
         g_free (path);
+ out:
+
+        if (pixbuf != NULL) {
+                framed = frame_pixbuf (pixbuf);
+                if (framed != NULL) {
+                        g_object_unref (pixbuf);
+                        pixbuf = framed;
+                }
+        }
 
         return pixbuf;
 }
