@@ -21,6 +21,8 @@
 #include "config.h"
 
 #include <string.h>
+#include <unistd.h>
+#include <sys/types.h>
 
 #include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
@@ -30,12 +32,11 @@
 #include <gconf/gconf-client.h>
 
 #include <glade/glade-xml.h>
+#include <dbus/dbus-glib.h>
 
 #include <bonobo/bonobo-main.h>
 #include <bonobo/bonobo-ui-util.h>
 
-#include <libgnome/gnome-init.h>
-
 #include <panel-applet.h>
 #include <panel-applet-gconf.h>
 
@@ -45,12 +46,12 @@
 #define LOCKDOWN_DIR    "/desktop/gnome/lockdown"
 #define LOCKDOWN_KEY    LOCKDOWN_DIR "/disable_user_switching"
 
-enum {
-        GDM_USER_STATUS_AVAILABLE,
-        GDM_USER_STATUS_BUSY,
-        GDM_USER_STATUS_INVISIBLE,
-        GDM_USER_STATUS_AWAY,
-};
+typedef enum {
+        GSM_PRESENCE_STATUS_AVAILABLE = 0,
+        GSM_PRESENCE_STATUS_INVISIBLE,
+        GSM_PRESENCE_STATUS_BUSY,
+        GSM_PRESENCE_STATUS_IDLE,
+} GsmPresenceStatus;
 
 typedef struct _GdmAppletData
 {
@@ -63,7 +64,9 @@ typedef struct _GdmAppletData
         GtkWidget      *menubar;
         GtkWidget      *menuitem;
         GtkWidget      *menu;
+#ifdef BUILD_PRESENSE_STUFF
         GtkWidget      *user_item;
+#endif
         GtkWidget      *control_panel_item;
         GtkWidget      *account_item;
         GtkWidget      *lock_screen_item;
@@ -80,6 +83,9 @@ typedef struct _GdmAppletData
         gint8           pixel_size;
         gint            panel_size;
         GtkIconSize     icon_size;
+#ifdef BUILD_PRESENSE_STUFF
+        DBusGProxy     *presence_proxy;
+#endif
 } GdmAppletData;
 
 typedef struct _SelectorResponseData
@@ -168,7 +174,7 @@ about_cb (BonoboUIComponent *ui_container,
         };
         char *license_i18n;
 
-        license_i18n = g_strjoinv ("\n\n", license);
+        license_i18n = g_strconcat (_(license[0]), "\n\n", _(license[1]), "\n\n", _(license[2]), NULL); 
 
         gtk_show_about_dialog (NULL,
                                "version", VERSION,
@@ -312,7 +318,7 @@ applet_change_background_cb (PanelApplet               *applet,
         gtk_widget_set_style (adata->menubar, NULL);
         rc_style = gtk_rc_style_new ();
         gtk_widget_modify_style (GTK_WIDGET (adata->menubar), rc_style);
-        gtk_rc_style_unref (rc_style);
+        g_object_unref (rc_style);
 
         switch (type) {
         case PANEL_NO_BACKGROUND:
@@ -476,6 +482,12 @@ gdm_applet_data_free (GdmAppletData *adata)
         g_signal_handler_disconnect (adata->user, adata->user_notify_id);
         g_signal_handler_disconnect (adata->user, adata->user_icon_changed_id);
 
+#ifdef BUILD_PRESENSE_STUFF
+        if (adata->presence_proxy != NULL) {
+                g_object_unref (adata->presence_proxy);
+        }
+#endif
+
         if (adata->user != NULL) {
                 g_object_unref (adata->user);
         }
@@ -546,7 +558,7 @@ menuitem_style_set_cb (GtkWidget     *menuitem,
                 const char *icon_name;
 
                 if (menuitem == adata->login_screen_item) {
-                        icon_name = "gdm";
+                        icon_name = "system-users";
                 } else if (menuitem == adata->lock_screen_item) {
                         icon_name = "system-lock-screen";
                 } else if (menuitem == adata->quit_session_item) {
@@ -728,12 +740,14 @@ on_manager_users_loaded (GdmUserManager *manager,
         update_switch_user (adata);
 }
 
+#ifdef BUILD_PRESENSE_STUFF
 static void
 on_user_item_activate (GtkMenuItem   *item,
                        GdmAppletData *adata)
 {
         g_signal_stop_emission_by_name (item, "activate");
 }
+#endif
 
 static void
 on_control_panel_activate (GtkMenuItem   *item,
@@ -872,6 +886,7 @@ on_quit_session_activate (GtkMenuItem   *item,
         g_free (args[0]);
 }
 
+#ifdef BUILD_PRESENSE_STUFF
 static gboolean
 on_menu_key_press_event (GtkWidget     *widget,
                          GdkEventKey   *event,
@@ -890,44 +905,70 @@ on_menu_key_press_event (GtkWidget     *widget,
 }
 
 static void
-set_status (GdmAppletData *adata,
-            guint          status)
+save_status (GdmAppletData *adata,
+             guint          status)
 {
-        adata->current_status = status;
-        update_label (adata);
+        if (adata->current_status != status) {
+                GError *error;
+
+                adata->current_status = status;
+
+                g_debug ("Saving status: %u", status);
+                error = NULL;
+                dbus_g_proxy_call (adata->presence_proxy,
+                                   "SetStatus",
+                                   &error,
+                                   G_TYPE_UINT, status,
+                                   G_TYPE_INVALID,
+                                   G_TYPE_INVALID);
+
+                if (error != NULL) {
+                        g_warning ("Couldn't save presence status: %s", error->message);
+                        g_error_free (error);
+                }
+        }
 }
 
 static void
 on_status_available_activate (GtkWidget     *widget,
                               GdmAppletData *adata)
 {
-        set_status (adata, GDM_USER_STATUS_AVAILABLE);
+
+        if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (widget))) {
+                save_status (adata, GSM_PRESENCE_STATUS_AVAILABLE);
+        }
 }
 
 static void
 on_status_busy_activate (GtkWidget     *widget,
                          GdmAppletData *adata)
 {
-        set_status (adata, GDM_USER_STATUS_BUSY);
+         if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (widget))) {
+                 save_status (adata, GSM_PRESENCE_STATUS_BUSY);
+         }
 }
 
 static void
 on_status_invisible_activate (GtkWidget     *widget,
                               GdmAppletData *adata)
 {
-        set_status (adata, GDM_USER_STATUS_INVISIBLE);
+         if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (widget))) {
+                 save_status (adata, GSM_PRESENCE_STATUS_INVISIBLE);
+         }
 }
 
 static struct {
-        char    *icon_name;
-        char    *display_name;
-        void    *menu_callback;
+        char      *icon_name;
+        char      *display_name;
+        void      *menu_callback;
+        GtkWidget *widget;
 } statuses[] = {
-        { "user-online", N_("Available"), on_status_available_activate },
-        { "user-busy", N_("Busy"), on_status_busy_activate },
-        { "user-invisible", N_("Invisible"), on_status_invisible_activate },
-        { "user-away", N_("Away"), NULL },
+        { "user-online", N_("Available"), on_status_available_activate, NULL },
+        { "user-invisible", N_("Invisible"), on_status_invisible_activate, NULL },
+        { "user-busy", N_("Busy"), on_status_busy_activate, NULL },
+        { "user-away", N_("Away"), NULL, NULL },
 };
+#endif
 
 static void
 update_label (GdmAppletData *adata)
@@ -937,25 +978,79 @@ update_label (GdmAppletData *adata)
 
         label = gtk_bin_get_child (GTK_BIN (adata->menuitem));
 
+#ifdef BUILD_PRESENSE_STUFF
         markup = g_strdup_printf ("<b>%s</b> <small>(%s)</small>",
-                                  gdm_user_get_real_name (GDM_USER (adata->user)),
-                                  statuses[adata->current_status].display_name);
+                                  gdm_user_get_display_name (GDM_USER (adata->user)),
+                                  _(statuses[adata->current_status].display_name));
+#else
+        markup = g_strdup_printf ("<b>%s</b>",
+                                  gdm_user_get_display_name (GDM_USER (adata->user)));
+#endif
         gtk_label_set_markup (GTK_LABEL (label), markup);
         g_free (markup);
 }
 
+#ifdef BUILD_PRESENSE_STUFF
+static void
+save_status_text (GdmAppletData *adata)
+{
+        GtkWidget     *entry;
+        GtkTextBuffer *buffer;
+        char          *escaped_text;
+        char          *text;
+        GtkTextIter    start, end;
+
+        entry = gdm_entry_menu_item_get_entry (GDM_ENTRY_MENU_ITEM (adata->user_item));
+        buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (entry));
+        gtk_text_buffer_get_bounds (buffer, &start, &end);
+        text = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
+        escaped_text = g_markup_escape_text (text, -1);
+
+        if (escaped_text != NULL) {
+                GError *error;
+
+                error = NULL;
+                dbus_g_proxy_call (adata->presence_proxy,
+                                   "SetStatusText",
+                                   &error,
+                                   G_TYPE_STRING, escaped_text,
+                                   G_TYPE_INVALID,
+                                   G_TYPE_INVALID);
+
+                if (error != NULL) {
+                        g_warning ("Couldn't set presence status text: %s", error->message);
+                        g_error_free (error);
+                }
+        }
+
+        g_free (text);
+        g_free (escaped_text);
+}
+
+static void
+on_user_item_deselect (GtkWidget     *item,
+                       GdmAppletData *adata)
+{
+        save_status_text (adata);
+}
+#endif
+
 static void
 create_sub_menu (GdmAppletData *adata)
 {
         GtkWidget *item;
+#ifdef BUILD_PRESENSE_STUFF
         int        i;
         GSList    *radio_group;
+#endif
 
         adata->menu = gtk_menu_new ();
+#ifdef BUILD_PRESENSE_STUFF
         g_signal_connect (adata->menu,
                           "key-press-event",
                           G_CALLBACK (on_menu_key_press_event),
                           adata);
+#endif
         gtk_menu_item_set_submenu (GTK_MENU_ITEM (adata->menuitem), adata->menu);
         g_signal_connect (adata->menu, "style-set",
                           G_CALLBACK (menu_style_set_cb), adata);
@@ -976,12 +1071,17 @@ create_sub_menu (GdmAppletData *adata)
                           G_CALLBACK (on_manager_user_added),
                           adata);
 
+#ifdef BUILD_PRESENSE_STUFF
         adata->user_item = gdm_entry_menu_item_new ();
         gtk_menu_shell_append (GTK_MENU_SHELL (adata->menu),
                                adata->user_item);
         gtk_widget_show (adata->user_item);
         g_signal_connect (adata->user_item, "activate",
                           G_CALLBACK (on_user_item_activate), adata);
+        g_signal_connect (adata->user_item,
+                          "deselect",
+                          G_CALLBACK (on_user_item_deselect),
+                          adata);
 
         item = gtk_separator_menu_item_new ();
         gtk_menu_shell_append (GTK_MENU_SHELL (adata->menu), item);
@@ -1001,7 +1101,7 @@ create_sub_menu (GdmAppletData *adata)
                 item = gtk_radio_menu_item_new (radio_group);
                 radio_group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (item));
                 hbox = gtk_hbox_new (FALSE, 3);
-                label = gtk_label_new (statuses[i].display_name);
+                label = gtk_label_new (_(statuses[i].display_name));
                 gtk_label_set_justify (GTK_LABEL(label), GTK_JUSTIFY_LEFT);
                 gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
                 gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
@@ -1017,11 +1117,14 @@ create_sub_menu (GdmAppletData *adata)
                 g_signal_connect (item, "activate",
                                   G_CALLBACK (statuses[i].menu_callback), adata);
                 gtk_widget_show (item);
+
+                statuses[i].widget = item;
         }
 
         item = gtk_separator_menu_item_new ();
         gtk_menu_shell_append (GTK_MENU_SHELL (adata->menu), item);
         gtk_widget_show (item);
+#endif
 
         adata->account_item = gtk_image_menu_item_new_with_label (_("Account Information..."));
         gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (adata->account_item),
@@ -1145,6 +1248,7 @@ reset_icon (GdmAppletData *adata)
                 return;
         }
 
+#ifdef BUILD_PRESENSE_STUFF
         if (adata->user_item != NULL) {
                 image = gdm_entry_menu_item_get_image (GDM_ENTRY_MENU_ITEM (adata->user_item));
                 pixbuf = gdm_user_render_icon (adata->user, adata->panel_size * 3);
@@ -1155,6 +1259,17 @@ reset_icon (GdmAppletData *adata)
                 gtk_image_set_from_pixbuf (GTK_IMAGE (image), pixbuf);
                 g_object_unref (pixbuf);
         }
+#else
+        pixbuf = gdm_user_render_icon (adata->user, adata->panel_size);
+
+        if (pixbuf == NULL) {
+                return;
+        }
+
+        image = gtk_image_menu_item_get_image (GTK_IMAGE_MENU_ITEM (adata->menuitem));
+        gtk_image_set_from_pixbuf (GTK_IMAGE (image), pixbuf);
+        g_object_unref (pixbuf);
+#endif
 }
 
 static void
@@ -1180,6 +1295,10 @@ setup_current_user (GdmAppletData *adata)
         }
 
         adata->menuitem = gtk_image_menu_item_new_with_label (name);
+#ifndef BUILD_PRESENSE_STUFF
+        gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (adata->menuitem),
+                                       gtk_image_new ());
+#endif
         label = GTK_BIN (adata->menuitem)->child;
         gtk_menu_shell_append (GTK_MENU_SHELL (adata->menubar), adata->menuitem);
         gtk_widget_show (adata->menuitem);
@@ -1202,6 +1321,61 @@ setup_current_user (GdmAppletData *adata)
         }
 }
 
+#ifdef BUILD_PRESENSE_STUFF
+static void
+set_status (GdmAppletData *adata,
+            guint status)
+{
+        int i;
+
+        g_debug ("Setting current status: %u", status);
+        adata->current_status = status;
+        for (i = 0; i < G_N_ELEMENTS (statuses); i++) {
+                if (statuses[i].widget == NULL) {
+                        continue;
+                }
+                if (i == status) {
+                        gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (statuses[i].widget),
+                                                        TRUE);
+                }
+        }
+
+        update_label (adata);
+}
+
+static void
+on_presence_status_changed (DBusGProxy    *presence_proxy,
+                            guint          status,
+                            GdmAppletData *adata)
+{
+        g_debug ("Status changed: %u", status);
+
+        set_status (adata, status);
+}
+
+static void
+set_status_text (GdmAppletData *adata,
+                 const char    *status_text)
+{
+        GtkWidget     *entry;
+        GtkTextBuffer *buffer;
+
+        g_debug ("Status text changed: %s", status_text);
+
+        entry = gdm_entry_menu_item_get_entry (GDM_ENTRY_MENU_ITEM (adata->user_item));
+        buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (entry));
+        gtk_text_buffer_set_text (buffer, status_text, -1);
+}
+
+static void
+on_presence_status_text_changed (DBusGProxy    *presence_proxy,
+                                 const char    *status_text,
+                                 GdmAppletData *adata)
+{
+        set_status_text (adata, status_text);
+}
+#endif
+
 static gboolean
 fill_applet (PanelApplet *applet)
 {
@@ -1215,6 +1389,8 @@ fill_applet (PanelApplet *applet)
         char              *tmp;
         BonoboUIComponent *popup_component;
         GdmAppletData     *adata;
+        GError            *error;
+        DBusGConnection   *bus;
 
         if (!first_time) {
                 first_time = TRUE;
@@ -1339,6 +1515,98 @@ fill_applet (PanelApplet *applet)
                 set_menu_visibility (adata, TRUE);
         }
 
+        error = NULL;
+        bus = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
+        if (bus == NULL) {
+                g_warning ("Unable to get session bus: %s", error->message);
+                g_error_free (error);
+                goto done;
+        }
+
+#ifdef BUILD_PRESENSE_STUFF
+        adata->presence_proxy = dbus_g_proxy_new_for_name (bus,
+                                                          "org.gnome.SessionManager",
+                                                          "/org/gnome/SessionManager/Presence",
+                                                          "org.gnome.SessionManager.Presence");
+        if (adata->presence_proxy != NULL) {
+                DBusGProxy *proxy;
+
+                dbus_g_proxy_add_signal (adata->presence_proxy,
+                                         "StatusChanged",
+                                         G_TYPE_UINT,
+                                         G_TYPE_INVALID);
+                dbus_g_proxy_connect_signal (adata->presence_proxy,
+                                             "StatusChanged",
+                                             G_CALLBACK (on_presence_status_changed),
+                                             adata,
+                                             NULL);
+                dbus_g_proxy_add_signal (adata->presence_proxy,
+                                         "StatusTextChanged",
+                                         G_TYPE_STRING,
+                                         G_TYPE_INVALID);
+                dbus_g_proxy_connect_signal (adata->presence_proxy,
+                                             "StatusTextChanged",
+                                             G_CALLBACK (on_presence_status_text_changed),
+                                             adata,
+                                             NULL);
+
+
+                proxy = dbus_g_proxy_new_from_proxy (adata->presence_proxy,
+                                                     "org.freedesktop.DBus.Properties",
+                                                     "/org/gnome/SessionManager/Presence");
+                if (proxy != NULL) {
+                        guint       status;
+                        const char *status_text;
+                        GValue      value = { 0, };
+
+                        status = 0;
+                        status_text = NULL;
+
+                        error = NULL;
+                        dbus_g_proxy_call (proxy,
+                                           "Get",
+                                           &error,
+                                           G_TYPE_STRING, "org.gnome.SessionManager.Presence",
+                                           G_TYPE_STRING, "status",
+                                           G_TYPE_INVALID,
+                                           G_TYPE_VALUE, &value,
+                                           G_TYPE_INVALID);
+
+                        if (error != NULL) {
+                                g_warning ("Couldn't get presence status: %s", error->message);
+                                g_error_free (error);
+                        } else {
+                                status = g_value_get_uint (&value);
+                        }
+
+                        g_value_unset (&value);
+
+                        error = NULL;
+                        dbus_g_proxy_call (proxy,
+                                           "Get",
+                                           &error,
+                                           G_TYPE_STRING, "org.gnome.SessionManager.Presence",
+                                           G_TYPE_STRING, "status-text",
+                                           G_TYPE_INVALID,
+                                           G_TYPE_VALUE, &value,
+                                           G_TYPE_INVALID);
+
+                        if (error != NULL) {
+                                g_warning ("Couldn't get presence status text: %s", error->message);
+                                g_error_free (error);
+                        } else {
+                                status_text = g_value_get_string (&value);
+                        }
+
+                        set_status (adata, status);
+                        set_status_text (adata, status_text);
+                }
+        } else {
+                g_warning ("Failed to get session presence proxy");
+        }
+#endif
+
+ done:
         gtk_widget_show (GTK_WIDGET (adata->applet));
 
         return TRUE;
